*****************************************************************************
	Subroutine HZ02023(iflag)
*****************************************************************************
* This routine produces plots from the H1 gaps between jets analysis :
* 
*     cross section in nano-barns
*
* Cuts : 	0.3 < y < 0.65
*		Et > 6.0,5.0 GeV
*               eta < 2.6
*               Q2 < 0.01 GeV^2
*               Delta eta > 2.5
* A gap event is defined as one in which there is a region of rapidity of at
* least 1.4 units of rapidity containing no object of energy > MeV between
* the two highest ET jet axis
*               
* Call the routine by:-
* 
* Call hz02023(1001) - initialisation (direct)
* Call hz02023(1002) - filling histos (direct)
* Call hz02023(1003) - termination (direct)
* Call hz02023(2001) - initialisation (resolved)
* Call hz02023(2002) - filling histos (resolved)
* Call hz02023(2003) - termination (resolved)
*
* 1st digit: Process: 1 = direct, 2 = resolved
* 3rd digit: Jetfinder: 2 = KTCLUS 
* 4th digit: Phase: 1 = initialisation, 2 = filling histos, 
*                   3 = termination
*
* Value for Ptmin is recommended to be set at 2.5 GeV (set in MC set up)
*
* Author : Brian Cox (Brian.Cox@cern.ch)
* Date   : 15 Jan 2002
*
* Reference : 
*
*
*****************************************************************************
*
* HERA Tuning defined commons
 
	IMPLICIT NONE
 
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
 
c *** ETGAP (figure 3a)
        REAL etgap(5),etgap_stat(5),etgap_sys_stat(5)
        DATA etgap/.122,.089,.141,.124,.054/
        DATA etgap_stat/.013,.006,.005,.003,.001/
        DATA etgap_sys_stat/.0,.0,.0,.0,.0/
	REAL etgap_bin(6),ETGAPBIN(5)
	DATA etgap_bin/0.00,0.5,1.50,3.50,7.00,12.00/
	DATA ETGAPBIN/0.5,1.45,2.0,3.50,5.0/


c *** DELTA ETA (figure 3b)
        REAL dleta(4),deta_stat(4),deta_sys_stat(4)
        DATA dleta/1.72,1.16,0.67,0.17/
        DATA deta_stat/0.04,0.03,0.02,0.01/
        DATA deta_sys_stat/.0,.0,.0,.0/
	REAL deta_bin(5),DETABIN(4)
	DATA deta_bin/2.5,2.8,3.1,3.5,4.0/
	DATA DETABIN/.3,.3,.4,.5/


c *** XGAMMA (figure 3b)
        REAL xgam(4),xgam_stat(4),xgam_sys_stat(4)
        DATA xgam/1.29,2.27,2.53,0.68/
        DATA xgam_stat/0.03,0.06,0.07,0.05/
        DATA xgam_sys_stat/.0,.0,.0,.0/
	REAL xgam_bin(5),XGAMBIN(4)
	DATA xgam_bin/0.3,0.6,0.75,0.90,1.00/
	DATA XGAMBIN/0.3,0.15,0.15,0.1/

c *** XPROTON (figure 3b)
        REAL xpro(4),xpro_stat(4),xpro_sys_stat(4)
        DATA xpro/22.3,23.2,8.6,2.8/
        DATA xpro_stat/0.5,0.6,0.3,0.2/
        DATA xpro_sys_stat/.0,.0,.0,.0/
	REAL xpro_bin(5),XPROBIN(4)
	DATA xpro_bin/0.02,0.04,0.06,0.08,0.10/
	DATA XPROBIN/0.02,0.02,0.02,0.02/

c *** DELTA ETA GAP FRACTIONS (figure 4)
	REAL dletag1(4),deta_statg1(4),deta_sys_statg1(4)
	REAL dletag2(4),deta_statg2(4),deta_sys_statg2(4)
	REAL dletag3(4),deta_statg3(4),deta_sys_statg3(4)
	REAL dletag4(4),deta_statg4(4),deta_sys_statg4(4)


        DATA dletag1/.050,.056,.025,.100/
        DATA deta_statg1/.006,.011,.009,.034/
        DATA deta_sys_statg1/.0,.0,.0,.0/

        DATA dletag2/.085,.079,.069,.101/
        DATA deta_statg2/.008,.011,.014,.029/
        DATA deta_sys_statg2/.0,.0,.0,.0/

        DATA dletag3/.137,.136,.099,.114/
        DATA deta_statg3/.010,.014,.014,.027/
        DATA deta_sys_statg3/.0,.0,.0,.0/

        DATA dletag4/.178,.167,.137,.189/
        DATA deta_statg4/.010,.014,.016,.034/
        DATA deta_sys_statg4/.0,.0,.0,.0/

c *** XGAMMA GAP FRACTIONS (figure 5)
	REAL xgamg1(4),xgam_statg1(4),xgam_sys_statg1(4)
	REAL xgamg2(4),xgam_statg2(4),xgam_sys_statg2(4)
	REAL xgamg3(4),xgam_statg3(4),xgam_sys_statg3(4)
	REAL xgamg4(4),xgam_statg4(4),xgam_sys_statg4(4)


        DATA xgamg1/.024,.046,.045,.277/
        DATA xgam_statg1/.007,.011,.008,.035/
        DATA xgam_sys_statg1/.0,.0,.0,.0/

        DATA xgamg2/.034,.060,.077,.462/
        DATA xgam_statg2/.007,.011,.010,.034/
        DATA xgam_sys_statg2/.0,.0,.0,.0/

        DATA xgamg3/.062,.079,.147,.623/
        DATA xgam_statg3/.009,.011,.013,.028/
        DATA xgam_sys_statg3/.0,.0,.0,.0/

        DATA xgamg4/.086,.098,.205,.712/
        DATA xgam_statg4/.011,.012,.013,.024/
        DATA xgam_sys_statg4/.0,.0,.0,.0/

c *** XPROTON GAP FRACTIONS (figure 6)
	REAL xprog1(4),xpro_statg1(4),xpro_sys_statg1(4)
	REAL xprog2(4),xpro_statg2(4),xpro_sys_statg2(4)
	REAL xprog3(4),xpro_statg3(4),xpro_sys_statg3(4)
	REAL xprog4(4),xpro_statg4(4),xpro_sys_statg4(4)


        DATA xprog1/.046,.044,.040,.034/
        DATA xpro_statg1/.007,.009,.016,.024/
        DATA xpro_sys_statg1/.0,.0,.0,.0/

        DATA xprog2/.102,.052,.054,.057/
        DATA xpro_statg2/.010,.008,.015,.024/
        DATA xpro_sys_statg2/.0,.0,.0,.0/

        DATA xprog3/.165,.083,.063,.057/
        DATA xpro_statg3/.012,.010,.014,.024/
        DATA xpro_sys_statg3/.0,.0,.0,.0/

        DATA xprog4/.202,.125,.118,.101/
        DATA xpro_statg4/.012,.011,.020,.033/
        DATA xpro_sys_statg4/.0,.0,.0,.0/


 
C --- cuts arrays
	DOUBLE PRECISION ycut(2),q2cut,detamax
	DOUBLE PRECISION etacut,Etcut(2),detacut,gapcut

C --- photoproduction cuts
        DATA q2cut/1.D-2/
	DATA ycut/0.3,0.65/
	DATA Etcut/6.,5./
	DATA etacut/2.65/
	DATA detacut/2.5/
	DATA detamax/4.0/
        
        CHARACTER*6 jetf

C --- information returned by jet finder
        INTEGER NJET,NJET_GOOD
        DOUBLE PRECISION Jet(50,8)
        DOUBLE PRECISION CONER
 
C --- private copies of Xsec and Ntot from HERACMN
	DOUBLE PRECISION mhXsec(3),mhNtot(3)
 
c ---
        DOUBLE PRECISION ye,q2,elecE,pflux
	INTEGER myil,myip
	INTEGER iproc, iflag, phase, chjet,chjetfind
	INTEGER hzflag

c ---   Analysis variables
	REAL rp(NMXHEP),en(NMXHEP),eppz,empz,rapmin,rapmax,dum,deta
	INTEGER Q
	REAL QR,rp2(NMXHEP),xp,eppzs,empzs,eppzj,empzj,xg,xg1
	INTEGER iobj,iloop,jloop,iround,iobj2,a,b,qcount,kloop,floop
	LOGICAL FLAG
	REAL et(NMXHEP),theta(NMXHEP),QET
	REAL profwgt(4)
	

	eppzs=0.
	empzs=0.
	eppzj=0.
	empzj=0.

C --- choose direct/resolved 
C --- recommend +1000 - DIRECT, +2000 - RESOLVED, +3000 - BFKL
C --- iproc = 1 --> DIRECT, iproc = 2 --> RESOLVED iproc = 3 --> BFKL 

        iproc = INT(iflag/1000)
 
C --- choose jet finder
c fixed to ktclus for now.
	chjetfind=9
        chjet=9

C --- 'phase' should be equal to 1,2,3, defines 
C --- initialisation, processing, termination phase respectively
        
        phase = INT(iflag-1000*iproc)

	IF (phase.eq.1) THEN
	   
           If (iproc.eq.1) then

C *** Initialisation run ***
C *** Create directories ***
	      CALL HCDIR('//HISTO',' ')
	      CALL HMDIR('02023','S')
	      CALL HCDIR('//PAWC',' ')
	      CALL HMDIR('02023','S')
	      
	      CALL HCDIR('//HISTO/02023',' ')
	      CALL HCDIR('//PAWC/02023',' ')     

C       ---Book Histograms
	      CALL jetprofile(1.D0,1.D0,100,101,phase)
	      
C       ---Cross sections as function of etgap
	      CALL HBOOKB(1,'BINWIDTH ETGAP',5,etgap_bin,0.)
	      CALL HBOOKB(-4001,'DATA ETGAP',5,etgap_bin,0.)  
	      CALL HBOOKB(1001,'ETGAP',5,etgap_bin,0.) 
	      CALL HBOOKB(2001,'RES ETGAP',5,etgap_bin,0.)

C       ---Cross sections as function of delta eta
	      CALL HBOOKB(2,'BINWIDTH DETA',4,deta_bin,0.)
	      CALL HBOOKB(-4002,'DATA DETA',4,deta_bin,0.)  
	      CALL HBOOKB(1002,'DETA',4,deta_bin,0.) 
	      CALL HBOOKB(2002,'RES DETA',4,deta_bin,0.)

C       ---Cross sections as function of xg
	      CALL HBOOKB(3,'BINWIDTH XGAM',4,xgam_bin,0.)
	      CALL HBOOKB(-4003,'DATA XGAM',4,xgam_bin,0.)  
	      CALL HBOOKB(1003,'XGAM',4,xgam_bin,0.) 
	      CALL HBOOKB(2003,'RES XGAM',4,xgam_bin,0.)


C       ---Cross sections as function of xp
	      CALL HBOOKB(4,'BINWIDTH XPRO',4,xpro_bin,0.)
	      CALL HBOOKB(-4004,'DATA XPRO',4,xpro_bin,0.)  
	      CALL HBOOKB(1004,'XPRO',4,xpro_bin,0.) 
	      CALL HBOOKB(2004,'RES XPRO',4,xpro_bin,0.)


C       ---Gap cross sections as a function of delta eta
	      CALL HBOOKB(1021,'DETA ETCUT 0.5',4,deta_bin,0.)
	      CALL HBOOKB(2021,'RES DETA ETCUT 0.5',4,deta_bin,0.)
	      CALL HBOOKB(-4025,'DATA DETA ETCUT 0.5',4,deta_bin,0.)

	      CALL HBOOKB(1022,'DETA ETCUT 1.0',4,deta_bin,0.)
	      CALL HBOOKB(2022,'RES DETA ETCUT 1.0',4,deta_bin,0.)
	      CALL HBOOKB(-4026,'DATA DETA ETCUT 1.0',4,deta_bin,0.)


	      CALL HBOOKB(1023,'DETA ETCUT 1.5',4,deta_bin,0.)
	      CALL HBOOKB(2023,'RES DETA ETCUT 1.5',4,deta_bin,0.)
	      CALL HBOOKB(-4027,'DATA DETA ETCUT 1.5',4,deta_bin,0.)


	      CALL HBOOKB(1024,'DETA ETCUT 2.0',4,deta_bin,0.)
	      CALL HBOOKB(2024,'RES DETA ETCUT 2.0',4,deta_bin,0.)
	      CALL HBOOKB(-4028,'DATA DETA ETCUT 2.0',4,deta_bin,0.)

C       ---Gap cross sections as a function of xg
	      CALL HBOOKB(1031,'XGAM ETCUT 0.5',4,xgam_bin,0.)
	      CALL HBOOKB(2031,'RES XGAM ETCUT 0.5',4,xgam_bin,0.)
	      CALL HBOOKB(-4035,'DATA XGAM ETCUT 0.5',4,xgam_bin,0.)

	      CALL HBOOKB(1032,'XGAM ETCUT 1.0',4,xgam_bin,0.)
	      CALL HBOOKB(2032,'RES XGAM ETCUT 1.0',4,xgam_bin,0.)
	      CALL HBOOKB(-4036,'DATA XGAM ETCUT 1.0',4,xgam_bin,0.)


	      CALL HBOOKB(1033,'XGAM ETCUT 1.5',4,xgam_bin,0.)
	      CALL HBOOKB(2033,'RES XGAM ETCUT 1.5',4,xgam_bin,0.)
	      CALL HBOOKB(-4037,'DATA XGAM ETCUT 1.5',4,xgam_bin,0.)


	      CALL HBOOKB(1034,'XGAM ETCUT 2.0',4,xgam_bin,0.)
	      CALL HBOOKB(2034,'RES XGAM ETCUT 2.0',4,xgam_bin,0.)
	      CALL HBOOKB(-4038,'DATA XGAM ETCUT 2.0',4,xgam_bin,0.)


C       ---Gap cross sections as a function of xp
	      CALL HBOOKB(1041,'XPRO ETCUT 0.5',4,xpro_bin,0.)
	      CALL HBOOKB(2041,'RES XPRO ETCUT 0.5',4,xpro_bin,0.)
	      CALL HBOOKB(-4045,'DATA XPRO ETCUT 0.5',4,xpro_bin,0.)

	      CALL HBOOKB(1042,'XPRO ETCUT 1.0',4,xpro_bin,0.)
	      CALL HBOOKB(2042,'RES XPRO ETCUT 1.0',4,xpro_bin,0.)
	      CALL HBOOKB(-4046,'DATA XPRO ETCUT 1.0',4,xpro_bin,0.)


	      CALL HBOOKB(1043,'XPRO ETCUT 1.5',4,xpro_bin,0.)
	      CALL HBOOKB(2043,'RES XPRO ETCUT 1.5',4,xpro_bin,0.)
	      CALL HBOOKB(-4047,'DATA XPRO ETCUT 1.5',4,xpro_bin,0.)


	      CALL HBOOKB(1044,'XPRO ETCUT 2.0',4,xpro_bin,0.)
	      CALL HBOOKB(2044,'RES XPRO ETCUT 2.0',4,xpro_bin,0.)
	      CALL HBOOKB(-4048,'DATA XPRO ETCUT 2.0',4,xpro_bin,0.)
	      
	      CALL HIDOPT(0,'stat')
	      CALL HBARX(0)
	   endif
           
           CALL HCDIR('//HISTO/02023',' ')
           CALL HCDIR('//PAWC/02023',' ')
	   
C       --- standard cone radius is 1.0 (for EUCELL)
C       --- cone radius not used for KTCLUS
           CALL HZJETRAD(2,CONER)
           IF (CONER.lt.0.0) THEN
              CONER=1.0
           ENDIF
           CALL HZJTNAME(chjetfind,jetf)
	   
	   WRITE(6,*)'**********************************************'
	   WRITE(6,*)'* HZ02023 called, histograms will be output. *'
	   WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	   WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	   WRITE(6,*)'**********************************************'
	   
 1001	   FORMAT(A29,F5.2,A13)

	ELSE IF (phase.eq.2) THEN

C       *** Filling runs ***
C       *** Change directory to our graphs ***
	   
           CALL HCDIR('//PAWC/02023',' ')
           
           q2=hzphokin(1)
           IF (q2.eq.-1) THEN
              PRINT*,'HZ02023: HZPHOKIN returns error for Q2'
           ELSEIF (q2.lt.0) THEN
              q2=abs(q2)
           ENDIF
           ye=hzphokin(3)
           IF (ye.eq.-1) THEN
              PRINT*,'HZ02023: HZPHOKIN returns error for y'
           ENDIF
	   
           IF ((ye.ge.ycut(1)).and.(ye.le.ycut(2))
     &          .and.(q2.lt.q2cut)) then 
              elecE=27.5
              hzflag=HZIBEAM(myip,myil)
              IF (hzflag.ge.0) then
                 elecE=abs(PHEP(4,myil))
              ELSE
                 PRINT*,'HZ02023: Electron not found. E=0.001'
                 PRINT*,'         event will be rejected'
                 elecE=0.001
              ENDIF
	      
C       --- Call Jetfinder 
	      
              CALL hzjtfind(chjetfind,CONER,NJET,Jet)
	      
              IF (NJET.ge.2) THEN
		 
C       --- Make jet cuts
C       --- Jets are ordered in Et by hzjtfind
                 If (Jet(1,3).ge.Etcut(1)     .and.
	1	      Jet(2,3).ge.Etcut(2)    .and.
	2	      Jet(1,1).lt.etacut      .and.
	3	      Jet(2,1).lt.etacut      .and.
     &                abs(Jet(1,1)-Jet(2,1)).gt.detacut .and.
     &                abs(Jet(1,1)-Jet(2,1)).lt.detamax) then
		    profwgt(iproc)=profwgt(iproc)+wtx
C		    CALL jetprofile(jet(1,1),jet(1,2),100,101,phase)

		    deta=abs(Jet(1,1)-Jet(2,1))
C       --- order jets, a most forward
		    a=1
		    b=2
		    if (Jet(2,1).gt.Jet(1,1)) then
		       a=2
		       b=1
		    endif
                    CALL jetprofile(jet(a,1),jet(a,2),100,101,phase)
C       --- sum eppz and empz jets
		    eppzj=REAL(Jet(1,4)+Jet(1,7)+Jet(2,4)+Jet(2,7))
		    empzj=REAL(Jet(1,4)-Jet(1,7)+Jet(2,4)-Jet(2,7))

		    QET=0.
c loop over jets and sum energy of all jets inbetween the 2 highest et jets
		    do iloop = 3,NJET,1
		       if (Jet(iloop,1).gt.Jet(b,1) .and.
	1		    Jet(iloop,1).lt.Jet(a,1)) then
			  QET=QET+REAL(Jet(iloop,3))
c			  write(*,*) Jet(iloop,1),Jet(iloop,3)
		       endif
		    enddo

C Fill etgap histograms

		    CALL HFILL(iproc*1000+1, QET ,0.,wtx)
		    CALL HPAK(1, ETGAPBIN)
		    CALL HPAK(-4001, etgap)
		    CALL HPAKE(-4001,etgap_stat)

C Fill delta eta histograms

		    CALL HFILL(iproc*1000+2, deta ,0.,wtx)
		    CALL HPAK(2, DETABIN)
		    CALL HPAK(-4002, dleta)
		    CALL HPAKE(-4002,deta_stat)

		    CALL HPAK(-4025, dletag1)
		    CALL HPAKE(-4025,deta_statg1)

		    CALL HPAK(-4026, dletag2)
		    CALL HPAKE(-4026,deta_statg2)


		    CALL HPAK(-4027, dletag3)
		    CALL HPAKE(-4027,deta_statg3)

		    CALL HPAK(-4028, dletag4)
		    CALL HPAKE(-4028,deta_statg4)
		    
	
		    xp=REAL((jet(1,3)*exp(1*jet(1,1))+jet(2,3)
	1		 *exp(1*jet(2,1))))
	2		 /(2*820.0)
		    xg=REAL((jet(1,3)*exp(-1*jet(1,1))+jet(2,3)
	1		 *exp(-1*jet(2,1))))
	2		 /(2*ye*elecE)

C Fill xgamma histograms

		    CALL HFILL(iproc*1000+3, xg ,0.,wtx)
		    CALL HPAK(3, XGAMBIN)
		    CALL HPAK(-4003, xgam)
		    CALL HPAKE(-4003,xgam_stat)

		    CALL HPAK(-4035, xgamg1)
		    CALL HPAKE(-4035,xgam_statg1)

		    CALL HPAK(-4036, xgamg2)
		    CALL HPAKE(-4036,xgam_statg2)


		    CALL HPAK(-4037, xgamg3)
		    CALL HPAKE(-4037,xgam_statg3)

		    CALL HPAK(-4038, xgamg4)
		    CALL HPAKE(-4038,xgam_statg4)

C Fill xproton histograms

		    CALL HFILL(iproc*1000+4, xp ,0.,wtx)
		    CALL HPAK(4, XPROBIN)
		    CALL HPAK(-4004, xpro)
		    CALL HPAKE(-4004,xpro_stat)

		    CALL HPAK(-4045, xprog1)
		    CALL HPAKE(-4045,xpro_statg1)

		    CALL HPAK(-4046, xprog2)
		    CALL HPAKE(-4046,xpro_statg2)


		    CALL HPAK(-4047, xprog3)
		    CALL HPAKE(-4047,xpro_statg3)

		    CALL HPAK(-4048, xprog4)
		    CALL HPAKE(-4048,xpro_statg4)
		    
		    
C Fill gap histograms for QET < 0.5, 1.0, 1.5, 2.0 GeV

		    Do iloop = 1,4,1
		       if (QET.lt.(0.5*float(iloop))) then
C			  write(*,*) 0.5*float(iloop),QET
			  CALL HFILL(iproc*1000+20+iloop,deta,0.,wtx)
			  CALL HFILL(iproc*1000+30+iloop,xg,0.,wtx)
			  CALL HFILL(iproc*1000+40+iloop,xp,0.,wtx)
		       endif
		    enddo
		 endif		! jet cuts
	      endif		! njet > 2
	   endif		! y and Q^2 cuts
	   
	   
	ELSE IF (phase.eq.3) THEN
	   
C       *** Termination run ***
C       *** Change directory ***
	   
	   CALL HCDIR('//PAWC/02023',' ')
	   
C       *** Finish off histograms ***
	   
	   IF (Xsec.eq.0) THEN
	      PRINT*
	1	   ,'HZ02023: termination called with zero cross section'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'          in process :',iproc*1000
	      Xsec=1
	   ENDIF
	   IF (Ntot.eq.0.0) THEN
	      PRINT*,'HZ02023: termination called with no total events'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'          in process :',iproc*1000
	      Ntot=1.0
	   ENDIF
	   
C       --- store Xsec and Ntot for direct, resolved and BFKL events.
	   mhXsec(iproc)=Xsec
	   mhNtot(iproc)=Ntot
	   
	   Print*,"HZ02023: iproc = ",iproc
	   Print*,"HZ02023: Xsec = ",Xsec
	   Print*,"HZ02023: Ntot = ",Ntot
	   
	   IF (iproc.eq.2) THEN




	      
	      
C       --- Normalise the direct, resolved, then add.
C       --- All non-normalised histograms are erased

	      DO jloop=1,2,1
		 write(*,*) jloop,mhXsec(jloop),mhNtot(jloop)
		 CALL HOPERA(1+jloop*1000,'+e',1+jloop*1000,
	1	      1+jloop*1000,
	2	      REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)		 
		 CALL HOPERA(2+jloop*1000,'+e',2+jloop*1000,
	1	      2+jloop*1000,
	2	      REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)
		 CALL HOPERA(3+jloop*1000,'+e',3+jloop*1000,
	1	      3+jloop*1000,
	2	      REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)
		 CALL HOPERA(4+jloop*1000,'+e',4+jloop*1000,
	1	      4+jloop*1000,
	2	      REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)
		 Do iloop=1,4,1
		    CALL HOPERA(jloop*1000+20+iloop,'+e',jloop*1000+20+iloop,
	1		 jloop*1000+20+iloop,
	2		 REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)
		    CALL HOPERA(jloop*1000+30+iloop,'+e',jloop*1000+30+iloop,
	1		 jloop*1000+30+iloop,
	2		 REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)
		    CALL HOPERA(jloop*1000+40+iloop,'+e',jloop*1000+40+iloop,
	1		 jloop*1000+40+iloop,
	2		 REAL(mhXsec(jloop)/(mhNtot(jloop))),0.)
		 Enddo		    
	      ENDDO

C       --- Add the direct and resolved histograms 
	      CALL HOPERA(1001,'+e',2001,4001,1.,1.)
	      CALL HOPERA(1002,'+e',2002,4002,1.,1.)
	      CALL HOPERA(1003,'+e',2003,4003,1.,1.)
	      CALL HOPERA(1004,'+e',2004,4004,1.,1.)

C       --- Add the direct and resolved gap histograms
	      Do iloop = 1,4,1
		 CALL HOPERA(1000+20+iloop,'+e',2000+20+iloop,4000+20+iloop,1.,1.)
		 CALL HOPERA(1000+30+iloop,'+e',2000+30+iloop,4000+30+iloop,1.,1.)
		 CALL HOPERA(1000+40+iloop,'+e',2000+40+iloop,4000+40+iloop,1.,1.)
	      Enddo
C       --- Divide to get gap fractions

	      Do iloop = 1,4,1
		 CALL HOPERA(4000+20+iloop,'/e',4002,4000+24+iloop,1.,1.)
		 CALL HOPERA(4000+30+iloop,'/e',4003,4000+34+iloop,1.,1.)
		 CALL HOPERA(4000+40+iloop,'/e',4004,4000+44+iloop,1.,1.)
	      Enddo


C       --- Divide cross sections by bin widths
	      CALL HOPERA(4001,'/e',1,4001,1.,1.)
	      CALL HOPERA(4002,'/e',2,4002,1.,1.)
	      CALL HOPERA(4003,'/e',3,4003,1.,1.)
	      CALL HOPERA(4004,'/e',4,4004,1.,1.)

C       --- Delete histograms
	      
	      do iloop=1,4,1
		 CALL HDELET(iloop)
		 do jloop=1,2,1
		    CALL HDELET(jloop*1000+iloop)
		    CALL HDELET(jloop*1000+20+iloop)
		    CALL HDELET(jloop*1000+30+iloop)
		    CALL HDELET(jloop*1000+40+iloop)
		 enddo
	      enddo
	      
	   ENDIF
C       *** End ***
	ENDIF
	
	RETURN
	END
	

