*CMZU:  0.00/05 30/01/96  18.28.04  by  Matthew Wing
*****************************************************************************
	Subroutine HZ98085(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is KTCLUS
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
*       These graph will be
*       meaningless unless Xsec and Ntot are set before
*       calling the termination routine.
*       (Xsec - total cross section returned by MC)
*       (Ntot - number of events passed to this routine)
*
* D* + jets measurement
*
* Cuts : 	Q2 < 1 GeV^2
*		0.19 < y_bj < 0.87
*     Jets:  ET1 > 7 GeV, ET2 > 6 GeV, |eta| < 2.4
*     Dstar: pt > 3 GeV , |eta| < 1.5
*
* Distribution:   id = 10, xgamma cross-section
* 
* Inclusive D* measurements.
*
* Cuts:       Q2 < 1 GeV^2
*           0.19 < y_bj < 0.87
*      Dstar: pt > 2 GeV , |eta| < 1.5
*
* Distributions:  id = 20, pT(D*)  cross-section
*                      32, eta(D*) cross-section for pT(D*) > 2 GeV
*                      33, eta(D*) cross-section for pT(D*) > 3 GeV
*                      34, eta(D*) cross-section for pT(D*) > 4 GeV
*                      35, eta(D*) cross-section for pT(D*) > 6 GeV
*
* Author : Matthew Wing (wing@mail.desy.de)
*
* (based on routine from Mark Hayes)
*
* Date   : 14/02/99
*
* Reference : DESY-98-085
*
*****************************************************************************
*
* HERA Tuning defined commons
 
	IMPLICIT NONE
 
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
 
	CHARACTER*6 jetf
 
        INTEGER NJET
        DOUBLE PRECISION Jets(50,8)
        DOUBLE PRECISION CONER
 
	DOUBLE PRECISION elecE
	INTEGER myil,myip
	INTEGER i,j,loop
	DOUBLE PRECISION xgam
	DOUBLE PRECISION px(2),py(2),pz(2),ptsq(2)
	DOUBLE PRECISION ppsq(2),partrap,rap(2),Et(2)
	DOUBLE PRECISION ybj,q2
 
C --- cuts arrays
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION rapcut(2),Etcut(5)
 
C --- private copies of Xsec and Ntot from HERACMN
	DOUBLE PRECISION mhxsec(2),mhntot(2)

	INTEGER intiflag, chjet, iproc, iflag
	SAVE chjet
	INTEGER hzflag
 
	REAL pttemp(12)
	INTEGER ivec,jvec
c	REAL sqrt
	real pttemporary(6), pttemperr(6)

	real ptdsbins(7)

C --- choose same jet finder as in the paper --- KTCLUS
	DATA chjet/3/
c --  binning for pT(D*) distribution
	data ptdsbins / 2., 3., 4., 5., 6., 8., 12. /
C *** Inclusive D* data
c
C *** pT cross-section
	real ptcr(6), ptsta(6), ptsysa(6), ptsysb(6)
	data ptcr   /  9.68,  4.94,  2.22,  1.076,  0.328,  0.067 /
        data ptsta  /  1.16,  0.31,  0.13,  0.073,  0.024,  0.008 /
        data ptsysa /  1.55,  0.30,  0.10,  0.071,  0.020,  0.004 /
	data ptsysb / -0.56, -0.31, -0.11, -0.043, -0.013, -0.006 /
C *** eta cross-section pT(D*) > 2GeV
	real eta2cr(6), eta2sta(6), eta2sysa(6), eta2sysb(6)
	data eta2cr   /  8.89,  8.16,  7.61,  5.23,  3.21,  4.65 /
        data eta2sta  /  0.81,  0.80,  0.79,  0.99,  1.02,  1.40 /
        data eta2sysa /  0.89,  0.70,  0.97,  1.09,  0.86,  1.49 /
	data eta2sysb / -0.33, -0.48, -0.48, -0.55, -0.66, -1.06 /
C *** eta cross-section pT(D*) > 3GeV
	real eta3cr(6), eta3sta(6), eta3sysa(6), eta3sysb(6)
	data eta3cr   /  2.96,  4.17,  3.88,  2.93,  2.11,  2.32 /
        data eta3sta  /  0.23,  0.29,  0.28,  0.28,  0.28,  0.34 /
        data eta3sysa /  0.24,  0.15,  0.25,  0.18,  0.19,  0.46 /
	data eta3sysb / -0.26, -0.26, -0.18, -0.25, -0.21, -0.37/
C *** eta cross-section pT(D*) > 4GeV
	real eta4cr(6), eta4sta(6), eta4sysa(6), eta4sysb(6)
	data eta4cr   /  1.021,  1.641,  1.877,  1.662,  1.090,  1.186 /
        data eta4sta  /  0.096,  0.137,  0.143,  0.137,  0.117,  0.142 /
        data eta4sysa /  0.079,  0.074,  0.117,  0.077,  0.138,  0.132 /
	data eta4sysb / -0.095, -0.118, -0.112, -0.067, -0.079, -0.137 /
C *** eta cross-section pT(D*) > 6GeV
	real eta6cr(6), eta6sta(6), eta6sysa(6), eta6sysb(6)
	data eta6cr   /  0.052,  0.331,  0.460,  0.398,  0.374,  0.242 /
        data eta6sta  /  0.052,  0.047,  0.061,  0.056,  0.045,  0.062 /
        data eta6sysa /  0.048,  0.054,  0.031,  0.032,  0.029,  0.030 /
	data eta6sysb / -0.037, -0.054, -0.039, -0.028, -0.036, -0.043 /

C *** Data for ET>7,6 GeV
	REAL cr(8),sta(8),sysa(8),sysb(8),banda(8),bandb(8)
C *** Actual data
	DATA cr    / 0.32, 1.06, 1.20, 0.98, 1.24, 1.80, 3.70, 2.87 /
	DATA sta   / 0.19, 0.30, 0.28, 0.31, 0.27, 0.36, 0.53, 0.37 /
	DATA sysa  / 0.14, 0.17, 0.17, 0.26, 0.33, 0.48, 0.61, 0.36 /
        DATA sysb  / -.26, -.22, -.36, -.26, -.24, -.20, -.65, -.33 /
	DATA banda / 0.00, 0.10, 0.17, 0.15, 0.18, 0.24, 0.54, 0.23 /
        DATA bandb / -.06, -.13, -.14, -.11, -.12, -.19, -.37, -.18 / 

C --- photoproduction cuts
 
	ybjcut(1)=0.19
	ybjcut(2)=0.87
	q2cut(1)=0.
	q2cut(2)=1.
	
C --- basic cuts for rapidity plots
	Etcut(1)=7.0		! first jet
	Etcut(2)=6.0		! second jet
	rapcut(1)=2.4
 
C --- choose direct/resolved
C --- recommend 1 - DIRECT, 2 - RESOLVED
	IF (iflag.gt.1000) THEN
	  iproc=INT(iflag/1000)
	  intiflag=MOD(iflag,1000)
	ENDIF
	IF ((iproc.gt.2).or.(iproc.lt.1)) THEN
	  PRINT*,'HZ98085 : Badly called routine'
	  PRINT*,'          please set intiflag +1000 or +2000'
	ENDIF
 
C --- choose jet finder
	IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
	  chjet=INT(intiflag/10)
	  intiflag=MOD(intiflag,10)
	ENDIF
	
	IF (intiflag.eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***
	  IF (iproc.eq.1) THEN
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR('98085','S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR('98085','S')
 	    call hbookb(20
     ,         , 'Monte Carlo pt(D*) cross-section (nb)'
     ,         , 6, ptdsbins, 0.)
	    call hbook1(32
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>2 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    call hbook1(33
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>3 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    call hbook1(34
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>4 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    call hbook1(36
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>6 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    CALL HBOOK1(10
     ,         ,'Monte Carlo xgamma cross-section (nb)'
     ,	       , 8, 0. ,1. ,0.)
	  ENDIF
	    CALL HCDIR('//HISTO/98085',' ')
	    CALL HCDIR('//PAWC/98085',' ')
 	    call hbookb(iproc*100+20
     ,         , 'Monte Carlo pt(D*) cross-section (nb)'
     ,         , 6, ptdsbins, 0.)
	    call hbook1(iproc*100+32
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>2 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    call hbook1(iproc*100+33
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>3 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    call hbook1(iproc*100+34
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>4 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    call hbook1(iproc*100+36
     ,         , 'Monte Carlo eta(D*) cross-section, pT(D*)>6 GeV (nb)'
     ,         , 6, -1.5, 1.5, 0.)
	    CALL HBOOK1(iproc*100+10
     ,         , 'Monte Carlo xgamma cross-section (nb)'
     ,         , 8, 0., 1., 0.)
 
	    CALL HIDOPT(0,'stat')
	    CALL HBARX(0)

C --- standard cone radius is 1.0
	  CALL HZJETRAD(2,CONER)
	  IF (CONER.lt.0.0) THEN
	    CONER=1.0
	  ENDIF
          CALL HZJTNAME(chjet,jetf)
 
	  WRITE(6,*)'**********************************************'
	  WRITE(6,*)'* HZ98085 called, histograms will be output. *'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	  WRITE(6,*)'**********************************************'
 
 1001	  FORMAT(A29,F5.2,A13)
 
	ELSE IF (intiflag.eq.2) THEN
C *** Filling runs ***
	
C *** Change directory to our graphs ***
	  CALL HCDIR('//PAWC/98085',' ')
 
	  q2=hzphokin(1)
	  IF (q2.eq.-1) THEN
	    PRINT*,'HZ98085: HZPHOKIN returns error for Q2'
	  ELSEIF (q2.lt.0) THEN
	    q2=abs(q2)
	  ENDIF
	  ybj=hzphokin(3)
	  IF (ybj.eq.-1) THEN
	    PRINT*,'HZ98085: HZPHOKIN returns error for ybj'
	  ENDIF
 
C *** Fill our histogram ***
 
	  IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &		.and.(q2.lt.q2cut(2))) Then
 
	  hzflag=HZIBEAM(myip,myil)
	  IF (hzflag.ge.0) Then
	     elecE=abs(PHEP(4,myil))
	  ELSE
	     PRINT*,'HZ98085: Electron not found. E=0.001'
	     PRINT*,'         event will be rejected'
	     elecE=0.001
	  ENDIF
	
	  DO i=1,NHEP
	    IF (abs(IDHEP(i)).eq.413) THEN !D*+-
	      px(1)=PHEP(1,i)
	      py(1)=PHEP(2,i)
	      pz(1)=PHEP(3,i)
	      ptsq(1)=px(1)**2+py(1)**2
	      ppsq(1)=(sqrt(ptsq(1)+pz(1)**2)+ABS(pz(1)))**2
	      IF (ptsq(1).le.4.25E-18*ppsq(1)) THEN
		partrap=20
	      ELSE
		partrap=0.5*LOG(ppsq(1)/ptsq(1))
	      ENDIF
	      partrap=SIGN(partrap,pz(1))
	      IF (abs(partrap).lt.1.5
     +       .and.ptsq(1).gt.4.0.and.ptsq(1).lt.144.0) THEN

		call hfill(iproc*100+20, real(sqrt(ptsq(1))),0.0, wtx)
		call hfill(iproc*100+32, real(partrap),0.0, wtx)

		if (ptsq(1).gt.9.0) then
		  call hfill(iproc*100+33, real(partrap), 0.0, wtx)
		  CALL hzjtfind(chjet,CONER,NJET,Jets)
		  IF (NJET.ge.2) THEN
c       Find the two highest ET jets in the allowed rapidity range.
		     j=0
		     DO loop=1,njet
c       Search jets in the rapidity interval.
			IF ( ABS(jets(loop,1)).lt.rapcut(1) ) THEN
			   j=j+1
			   IF (j.LE.2) THEN
			      Et(j)=Jets(loop,3)
			      rap(j)=Jets(loop,1)
			   ENDIF
			ENDIF
		     ENDDO

		    IF (Et(1).gt.Etcut(1).and.Et(2).gt.Etcut(2)) THEN
		      xgam=(Et(1)*exp(-1*rap(1))+Et(2)*exp(-1*rap(2)))
     +		          /(2*ybj*elecE)
		      CALL HFILL(iproc*100+10,REAL(xgam),0.0,wtx)
		    end if
		  end if
	        end if
		if (ptsq(1).gt.16.0) then
		  call hfill(iproc*100+34, real(partrap),0.0,wtx)
	        end if
		if (ptsq(1).gt.36.0) then
		  call hfill(iproc*100+36, real(partrap), 0.0,wtx)
	        end if
	      ENDIF		! momentum and eta cuts
	    ENDIF		! Dstar
	  ENDDO			! loop over HEPEVTP
 
	  ENDIF		! Q2 & y cuts
 
	ELSE IF (intiflag.eq.3) THEN
C *** Termination run ***
C *** Change directory ***
	  CALL HCDIR('//PAWC/98085',' ')
C *** Finish off histograms ***
 
	  IF (Xsec.eq.0) THEN
	    PRINT*,'HZ98085: termination called with zero cross section'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Xsec=1
	  ENDIF
	  IF (Ntot.eq.0) THEN
	    PRINT*,'HZ98085: termination called with no total events'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Ntot=1
	  ENDIF
 
C --- store Xsec and Ntot for both direct and resolved events.
	  if (iproc.eq.1) then
	     mhXsec(1)=Xsec
	     mhntot(1)=Ntot
	     CALL HOPERA(120, '+e', 120, 20,
     &		 REAL(mhXsec(1)/mhNtot(1)), 0.)
	     call hunpak(20, pttemporary, ' ', 0)
	     call hunpke(20, pttemperr, ' ', 0)
c	     do i = 1, 6
c		pttemporary(i) = pttemporary(i)/(ptdsbins(i+1)-ptdsbins(i))
c		pttemperr(i)   = pttemperr(i)/(ptdsbins(i+1)-ptdsbins(i))
c	     end do
	     call hpak(20, pttemporary)
	     call hpake(20, pttemperr)
	     CALL HOPERA(132, '+e', 132, 32,
     &		 REAL(2*mhXsec(1)/mhNtot(1)), 0.)
	     CALL HOPERA(133, '+e', 133, 33,
     &		 REAL(2*mhXsec(1)/mhNtot(1)), 0.)
	     CALL HOPERA(134, '+e', 134, 34,
     &		 REAL(2*mhXsec(1)/mhNtot(1)), 0.)
	     CALL HOPERA(136, '+e', 136, 36,
     &		 REAL(2*mhXsec(1)/mhNtot(1)), 0.)
	     CALL HOPERA(110, '+e', 110, 10,
     &		 REAL(8*mhXsec(1)/mhNtot(1)), 0.)
	  end if

	  IF (iproc.eq.2) THEN
C --- only finally
C --- create proper cross-section data
	    call hbookb(-20, 'Data pt(D*) cross-section (nb)'
     ,                     , 6, ptdsbins, 0.)
	    call hbook1(-32, 'Data eta(D*) cross-section, pT(D*)>2 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    call hbook1(-33, 'Data eta(D*) cross-section, pT(D*)>3 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    call hbook1(-34, 'Data eta(D*) cross-section, pT(D*)>4 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    call hbook1(-36, 'Data eta(D*) cross-section, pT(D*)>6 GeV (nb)'
     ,                     , 6, -1.5, 1.5, 0.)
	    CALL HBOOK1(-10,'Data xgamma cross-section (nb)'
     ,	                   , 8, 0., 1., 0.)
 
	    call hpak(-20, ptcr)
	    call hpak(-32, eta2cr)
	    call hpak(-33, eta3cr)
	    call hpak(-34, eta4cr)
	    call hpak(-36, eta6cr)
	    CALL HPAK(-10, cr)

	    call hpake(-20, ptsta)
	    call hpake(-32, eta2sta)
	    call hpake(-33, eta3sta)
	    call hpake(-34, eta4sta)
	    call hpake(-36, eta6sta)
	    CALL HPAKE(-10, sta)
 
C --- now calculate cross-section graphs 10,20

	    mhXsec(2)=Xsec
	    mhntot(2)=Ntot

	    CALL HOPERA(20, '+e', 220, 20,
     &		 1.,   REAL(mhXsec(2)/mhNtot(2)))
	    call hunpak(20, pttemporary, ' ', 0)
	    call hunpke(20, pttemperr, ' ', 0)
	    do i = 1, 6
	       pttemporary(i) = pttemporary(i)/(ptdsbins(i+1)-ptdsbins(i))
	       pttemperr(i)   = pttemperr(i)/(ptdsbins(i+1)-ptdsbins(i))
	    end do
	    call hpak(20, pttemporary)
	    call hpake(20, pttemperr)
	    CALL HOPERA(32, '+e', 232, 32,
     &		 1., REAL(2*mhXsec(2)/mhNtot(2)))
	    CALL HOPERA(33, '+e', 233, 33,
     &		 1., REAL(2*mhXsec(2)/mhNtot(2)))
	    CALL HOPERA(34, '+e', 234, 34,
     &		 1., REAL(2*mhXsec(2)/mhNtot(2)))
	    CALL HOPERA(36, '+e', 236, 36,
     &		 1., REAL(2*mhXsec(2)/mhNtot(2)))
	    CALL HOPERA(10, '+e', 210, 10,
     &		 1., REAL(8*mhXsec(2)/mhNtot(2)))
c
     	    CALL HZCHISQ(-10,10)
     	    CALL HZCHISQ(-20,20)
     	    CALL HZCHISQ(-32,32)
     	    CALL HZCHISQ(-33,33)
     	    CALL HZCHISQ(-34,34)
     	    CALL HZCHISQ(-36,36)
	  ENDIF
 
	ELSE
C *** End ***
	  PRINT*,'HZ98085: Please run routine with iflag set to 1,2 or 3'
	  PRINT*,'         with +1000 or +2000 for DIR or RES events.'
	ENDIF
 
	RETURN
	END
