      PROGRAM PYT
C
C     Pythia main program for JetWeb.
C
      IMPLICIT NONE

c     HBOOK stuff
      INTEGER NWPAWC,H
      PARAMETER (NWPAWC=9000000)
      COMMON/PAWC/H(NWPAWC)

c     Counter used in event loop
      INTEGER EVLOOP
      INTEGER ILOOP

c     Total cross section
      REAL XSECLOC

c     Mean charged multiplicity
      DOUBLE PRECISION nchavg

c     Temporary variables for output of HZNEVT
      INTEGER NEV2
      REAL LUMI2

c     HEPEVT prime common block
#include "hepevtp.inc"

c     Misleadingly named common block
c     Contains information on generator, no. of events etc.
#include "heracmn.inc"

c     Pythia common blocks
#include "hzpytsteer.inc"

c     Common block with steering parameters
#include "hzcontrol.inc"
#include "hzpytcontrol.inc"

c     Move to common block, e.g. HZFUNC?
      DOUBLE PRECISION HZPYAVRG

      PRINT*,''
      PRINT*,'*******************************************************'
      PRINT*,'*'
      PRINT*,'* pyt_test.fpp $Date: 2004/08/27 17:13:59 $'  
      PRINT*,'*'
      PRINT*,'*******************************************************'
      PRINT*,''

      GEN = 'PYT'

c     Read steering cards etc.
      CALL HZCARDS(1)
      CALL HZPYTCARDS
      CALL HZCARDS(2)

c     Set beam energies
      P(1,1)=0.
      P(1,2)=0.
      P(1,3)=ebeam1
      P(2,1)=0.
      P(2,2)=0.
      P(2,3)=-ebeam2

c     Set minimum PT for hard subprocess
      ckin(3) = ptmin

c     Copy MSTP, PARP, MSTJ and PARJ from steering cards
      DO 22 iloop = 1, 200
        IF (HZPARP(ILOOP).GT.-9999.) PARP(ILOOP) = HZPARP(ILOOP)
        IF (HZMSTP(ILOOP).GT.-9999.) MSTP(ILOOP) = HZMSTP(ILOOP)
        IF (HZPARJ(ILOOP).GT.-9999.) PARJ(ILOOP) = HZPARJ(ILOOP)
        IF (HZMSTJ(ILOOP).GT.-9999.) MSTJ(ILOOP) = HZMSTJ(ILOOP)
 22   CONTINUE

c     Set initial seed for random number generator
      mrpy(1)=rand_num1
      WRITE(*,*) 'Using random number sequence ',mrpy(1)

c     Initialize Pythia for p/pbar collisions with beam 4-momenta
c     given by array P. Last argument is dummy.
      CALL PYINIT('3MOM',beam1,beam2,0.D0)

c ??? Unfortunately need to do dodgy things like switch off pi0 decays
c     here for certain process types / routines.
c     Or maybe leave this until after initialization for main event loop?

c     Generate a few events to estimate cross section
      write (6,*) 'Generating a few events to estimate cross section'
      DO EVLOOP=1,100
        CALL PYEVNT
      ENDDO
      CALL PYSTAT(1)

c     Get total cross section from Pythia in millibarns. Convert to pb.
      XSECLOC = oldXSEC(0,3)*1E+09

c     Decide how many events to generate and corresponding luminosity
      CALL HZNEVT(XSECLOC, NEV, LUMI, NEV2, LUMI2)
      NEV = NEV2

c     Calculate mean charged multiplicity, needed by some analysis routines
      write (6,*)
     &      'Generating a few events to estimate mean multiplicity'
      nchavg = HZPYAVRG(5000)

c     Re-initialize Pythia to clear lumi and number of events etc.
      CALL PYINIT('3MOM',BEAM1,BEAM2,0.D0)

c ??? Unfortunately need to do dodgy things like switch off pi0 decays
c     here for certain process types / routines.

c     Write our some numbers to the log file:-
c     User-Defined Limits on Kinematical Variables
      CALL PYSTAT(4)
c     Summary of Status Codes and Parameter Values (common /PYPARS/)
      CALL PYSTAT(5)
c     List parameter value table (common /PYDAT1/, /PYDAT2/)
      CALL PYLIST(13)

c     Initialize HBOOK histogramming
      CALL HZHISTO(1,nchavg)

c     Generate events
      write (6,*) 'Starting main event generation loop'
      DO EVLOOP=1,NEV
c       Call Pythia to generate event
        CALL PYEVNT
c       Transfer information to HEPEVT common, HERACMN etc.
        CALL HZFILHEP
c       Write out first couple of events
        IF (EVLOOP.LE.2) CALL PYLIST(1)
c       Fill histograms
        CALL HZHISTO(2,nchavg)
      ENDDO

c     Print out some info from Pythia
      CALL PYSTAT(1)   

c     Finish off HBOOK stuff
      CALL HZHISTO(3,nchavg)

      PRINT*,''
      PRINT*,'*******************************************************'
      PRINT*,'*'
      PRINT*,'* Outputted and ended'    
      PRINT*,'*'
      PRINT*,'*******************************************************'
      PRINT*,''

      END
