      PROGRAM PPRUN
c     ------------------------------------------------------------
c     
c     Main program to make proton-proton and proton-antiproton plots
c     for JetWeb
c
c     Each CMS energy requires a reinitialisation of HERWIG.
c     Currently there are runs for: 
c     Tevatron 1800 GeV ppbar
c     Tevatron 1800 GeV ppbar with stable pi0, K0s, Lambda0
c     SPS 200 GeV ppbar
c     SPS 546 GeV ppbar
c     SPS 900 GeV ppbar
c
c     If weighted events are generated, the number of events is
c     the same for each run and is determined by the NEV input card.
c     If unweighted events are generated, the the number of events
c     for the Tevatron is calculated from the cross section and the
c     LUMI card, and for the SPS runs is set to 10 times the number
c     for the Tevatron.
c  
c     Author: Jon Butterworth, J.Butterworth@ucl.ac.uk
c 
c     ------------------------------------------------------------
* HERWIG/JIMMY COMMONS. HERWIG COMMON INCLUDES AN IMPLICIT NONE.
#ifdef HERWIG64
#include "herwig64.inc"
#else
#include "herwig.inc"
#endif
#include "jimmy30.inc"

* HZTOOL COMMONS
#include "heracmn.inc"
#include "hzhbook.inc"
	 
      DOUBLE PRECISION avrg, hzhwavrg

      LOGICAL ABORT, premserr

C     HBOOK common.
      INTEGER NWPAWC,H,loop3,loop4
      PARAMETER (NWPAWC=200000)
      COMMON/PAWC/H(NWPAWC)
      
C     INTEGER istat
      INTEGER icycle,loop      
      INTEGER n,evloop,i,itest,UP

      INTEGER status
      DOUBLE PRECISION dummy,hwrset

      REAL tot_qcd

C     Variables for parameter setting etc.
      INTEGER ipunit,lenocc
      PARAMETER(ipunit=5)
      CHARACTER*80 stem

#include "hzcontrol.inc"

      DOUBLE PRECISION E_max(100),ivy(100),event,cs,NRG,ceta
      REAL const1,const2,const3,const4,const5,norm(16)

C     HBOOK memory allocation.
      CALL HLIMIT(nwpawc)

C     Start qcd events.
C     -----------------------------
      IPROC=11500


C     Choose generator.
      GEN='HRW'
 
      PART1='P  '
      PART2='PBAR  '

      tot_qcd = 0.0
      rand_num1=0
      rand_num2=0

      WRITE(*,*) 'File stem?'
      READ(*,3000) stem
 3000 FORMAT(A80)
      stem = stem(1:LENOCC(stem))

      WRITE(*,*) 'Reading from ',stem(1:LENOCC(stem))//'.input'

      OPEN(UNIT=ipunit,FILE=stem(1:LENOCC(stem))//'.input',STATUS
     &     ='UNKNOWN')

C     Open up our histogram file.
C      CALL HROPEN(45,'HISTO',STEM(1:LENOCC(STEM))//'.rz','N'
C     &     ,1024,status
C     &     )
C      WRITE(*,*),'Histogram file opened:',stem//'.rz'

c     Read in control cards.
      CALL HZCARDS

      IPROC=11500

***********************************
**  Start of first Tevatron run  **
***********************************
      
      PBEAM1=7000
      PBEAM2=7000
      tot_qcd = 0.0
C     HERWIG initialization
      NUMER=0
      CALL HWIGIN
      
C     These parameters must be set here!
#include "hrwparam_tevatron.inc"

      IF (MION.GT.1) THEN
         WRITE(*,*) 'SOFT UNDERLYING EVENT IS TURNED ON'
         IPROC=MOD(IPROC,10000)
      ENDIF

      CALL HWUINC

      IF (MSFLAG.NE.0) THEN
         IF (.NOT.NOWGT) THEN
c     Need to run the search for maximum weight regardless.
            NOWGT=.TRUE.
         ENDIF
      ENDIF
      
      CALL HWEINI

c     Reinstate the users choice of weighting or not.
      NOWGT=.NOT.WEIGHTING

c      IF (NOWGT) THEN
c     Luminosity requested defaults to something sensible.
c         IF (hzlumi.LT.0) hzlumi=1.0E-5
         
c         NEV_QCD = AVWGT*HZLUMI*1000.
c         WRITE(*,*) 'Lumi to generate (pb-1): ',HZLUMI
c      ENDIF

      WRITE(*,*) 'Number of events to generate =',NEV_QCD

      MAXEV=NEV_QCD
      MAXER=MAXEV

      NRN(1)=RAND_NUM1
      NRN(2)=RAND_NUM2
      DUMMY=HWRSET(NRN)

      WRITE(*,2000) PTMIN,MODPDF(1),AUTPDF(1)
     &     ,MODPDF(2),AUTPDF(2),PTPOW,MSFLAG,STEM
 2000   FORMAT(
     &       'Minumum PT ',F10.3/,
     &       'PDF',I6,A6,I6,A6,/,
     &       'PTPOW',F10.2/,
     &       'MI on',I6/,
     &       'File name: ',A30)
C     Random number
      WRITE(*,*) 'Using seeds',NRN(1),NRN(2)
      IF (.NOT.NOWGT) WRITE(*,*) 'Weighted Events'

C     Histogram initialization for resolved.
      PRINT*,"Initialised QCD Scattering"

      call hzgbjlhc(1)

C     Loop over events.

      DO N=1,MAXEV
 50      CONTINUE
  
C---  INITIALISE EVENT
         CALL HWUINE
C---    GENERATE HARD SUBPROCESS
         CALL HWEPRO
C---  GENERATE PARTON CASCADES
         CALL HWBGEN
         premserr=.false.
         if (ierror.eq.0) premserr=.true.
C---  GENERATE MULTIPARTON INTERACTIONS
         IF (MSFLAG.EQ.1) THEN
            CALL HWMSCT(ABORT)
            IF (ABORT) GOTO 50
         ENDIF

C---  DO HEAVY OBJECT DECAYS
#ifdef HERWIG59 
         CALL HWDHQK
#else
         CALL HWDHOB
#endif
C---  DO CLUSTER HADRONIZATION
         CALL HWCFOR
C---  DO CLUSTER DECAY
         CALL HWCDEC
C---  DO UNSTABLE PARTICLE DECAYS
         CALL HWDHAD
C---  DO HEAVY FLAVOUR DECAYS
         CALL HWDHVY
C---  ADD SOFT UNDERLYING EVENT IF NEEDED
         CALL HWMEVT
C---  FINALISE EVENT
         CALL HWUFNE
         
         IF (IERROR.NE.0) GOTO 50
         
         CALL HZFILHEP
         
         IF (MOD(n,1000).eq.0) THEN
            PRINT*,"QCD event ",n
         ENDIF
         IF (NOWGT) THEN
            WTX=1.0
         ELSE
            WTX=EVWGT
*            write(*,*) evwgt,wtx
         ENDIF
         
         tot_qcd=tot_qcd+wtx

c     Final state particles
         CALL hzgbjlhc(2)

         
      ENDDO
      
C     Finish off Herwig Routines
      CALL HWEFIN
      
C     Close histograms.
      Ntot=tot_qcd
      Xsec=AVWGT
      
c     Final state particles
      CALL hzgbjlhc(3)

      PRINT*,"Terminated Run"



************************
**  End of final run  **
************************      

C      CALL HCDIR('//PAWC',' ')
C      CALL HCDIR('//HISTO',' ')
C      CALL HROUT(0,icycle,'T')
C      CALL HREND('HISTO')
      
        
      PRINT*,"Outputted and ended"	
      
      END

      SUBROUTINE hwaend
      RETURN
      END

















